<?php

namespace Drupal\monarch_migration\Plugin\MigrationMapper;

use Drupal\migrate\Row;
use Drupal\Core\Plugin\PluginBase;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\monarch_migration\MigrationMapperInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The migration mapper plugin base class.
 */
abstract class MigrationMapperBase extends PluginBase implements MigrationMapperInterface, ContainerFactoryPluginInterface {

  /**
   * The current database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The legacy database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $sourceDatabase;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );

    $instance->database = $container->get('database');

    if (
      $configuration['migration'] &&
      method_exists($configuration['migration'], 'getSourcePlugin') &&
      ($source_plugin = $configuration['migration']->getSourcePlugin()) &&
      method_exists($source_plugin, 'getDatabase')
    ) {
      $instance->sourceDatabase = $source_plugin->getDatabase();
    }

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row, MigrationInterface $migration) {}

  /**
   * {@inheritdoc}
   */
  public function postProcess(Row $row, MigrationInterface $migration) {}

  /**
   * {@inheritdoc}
   */
  public function preRowSave(Row $row, MigrationInterface $migration) {}

  /**
   * {@inheritdoc}
   */
  public function postRowSave(Row $row, MigrationInterface $migration) {}

  /**
   * {@inheritdoc}
   */
  public function getMigrations() : array {
    $ret = $this->pluginDefinition['migration'] ?? [];

    if (!is_array($ret)) {
      $ret = [$ret];
    }

    return array_filter($ret);
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(string $migration_id) {
    return in_array($migration_id, $this->getMigrations());
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight() {
    $this->pluginDefinition['weight'] ?? 0;
  }

}
